use crate::cpu::{instructions::Instruction, model::CPU};
use crate::msl_result::{MSLError, MSLResult};
use crate::ram::model::RAM;

impl CPU {
    pub fn init(alloc: usize) -> CPU {
        CPU {
            memory: RAM::new(alloc),
            ..Default::default()
        }
    }

    pub fn store_instructions(
        &mut self,
        instructions: Vec<Instruction>,
    ) -> MSLResult<()> {
        if instructions.len() + 10 > self.memory.mem.len() {
            return Err(MSLError::MemoryError(
                format!("Not enough allocated memory to store the whole program. Minimal required memory is {} cells", instructions.len() + 10)
            ).into());
        }
        (&mut self.memory).store_instructions(instructions)
    }

    pub fn free(&mut self) -> MSLResult<()> {
        (&mut self.memory).free()
    }
}
